unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, IntfGraphics,
  StdCtrls, Spin;

type
  
  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Label1: TLabel;
    PaintBox1: TPaintBox;
    SpinEdit1: TSpinEdit;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    FFraction: Double;
    FImage1: TLazIntfImage;
    FImage2: TLazIntfImage;
    FImage: TLazIntfImage;
    FBitmap: TBitmap;
    FDirection: Boolean;
    FTime: TDateTime;
    FFrames: Integer;

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

uses
  fpimage;

const 
  IMG_FILENAME_1 = 'cheetah.jpg';
  IMG_FILENAME_2 = 'cheetah-gespiegelt.jpg';

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  jpg: TJpegImage;
begin
  jpg := TJpegImage.Create;
  try
    jpg.LoadFromFile(IMG_FILENAME_1);
    FImage1 := jpg.CreateIntfImage;
  finally
    jpg.Free;
  end;
  
  jpg := TJpegImage.Create;
  try
    jpg.LoadFromFile(IMG_FILENAME_2);
    FImage2 := jpg.CreateIntfImage;
  finally
    jpg.Free;
  end;
  
  if (FImage1.Height <> FImage2.Height) or (FImage1.Width <> FImage2.Width) then
  begin
    MessageDlg('Bildgrößen sind unterschiedlich', mtError, [mbOK], 0);
    Close;
  end;
  
  FBitmap := TBitmap.Create;
  FBitmap.LoadFromIntfImage(FImage1);
  
  FImage := FBitmap.CreateIntfImage;
  
  Paintbox1.Width := FBitmap.Width;
  Paintbox1.Height := FBitmap.Height;
  
  SpinEdit1.Value := Timer1.Interval;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FFrames := 0;

  FDirection := not FDirection;
  if FDirection then
    FFraction := 0
  else
    FFraction := 1.0;
  Paintbox1.Invalidate;
  
  FTime := Now;
  Caption := 'Running...';
  Timer1.Enabled := true;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FBitmap.Free;
  FImage.Free;
  FImage1.Free;
  FImage2.Free;
end;

procedure TForm1.PaintBox1Paint(Sender: TObject);
var
  x, y: Integer;
  r, g, b: Word;
begin
  inc(FFrames);
  
  for y := 0 to FBitmap.Height-1 do
    for x := 0 to FBitmap.Width - 1 do
    begin
      r := round((1.0-FFraction) * FImage1.Colors[x, y].Red + FFraction * FImage2.Colors[x, y].Red);
      g := round((1.0-FFraction) * FImage1.Colors[x, y].Green + FFraction * FImage2.Colors[x, y].Green);
      b := round((1.0-FFraction) * FImage1.Colors[x, y].Blue + FFraction * FImage2.Colors[x, y].Blue);
      FImage.Colors[x, y] := FPColor(r, g, b);
    end;
  
  FBitmap.LoadFromIntfImage(FImage);
  Paintbox1.Canvas.Draw(0, 0, FBitmap);
end; 

procedure TForm1.SpinEdit1Change(Sender: TObject);
begin
  Timer1.Interval := SpinEdit1.Value;
end; 

procedure TForm1.Timer1Timer(Sender: TObject);
const
  DELTA = 0.05;
begin
  if FDirection then
  begin
    FFraction := FFraction + DELTA;
    if FFraction >= 1.0 then
    begin
      Timer1.Enabled := false;
      FTime := Now - FTime;
      Caption := Format('%s seconds, %d frames --> %s seconds/frame', [
        FormatDateTime('s.zzz', FTime), FFrames, FormatDateTime('s.zzz', FTime/FFrames)]);
    end;
  end else
  begin
    FFraction := FFraction - DELTA;
    if FFraction <= 0.0 then
    begin
      Timer1.Enabled := false;
      FTime := Now - FTime;
      Caption := Format('%s seconds, %d frames --> %s seconds/frame', [
        FormatDateTime('s.zzz', FTime), FFrames, FormatDateTime('s.zzz', FTime/FFrames)]);
    end;
  end;
  Paintbox1.Invalidate;
end; 

end.

